using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Log;

namespace WingServerCommon.Config
{
    /// <summary>
    /// This class will load system environment config data
    /// </summary>
    public class EnvironmentConfigManager
    {
        private static ConcurrentDictionary<string, IParameter> _parameters;

        /// <summary>
        /// Init load appsetting
        /// </summary>
        static EnvironmentConfigManager()
        {
            var configFilePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "appsettings.json");
            _parameters = ParameterDictionarySerializer.Deserialize(configFilePath);
        }

        /// <summary>
        /// Add or Update Config File
        /// </summary>
        /// <param name="filePath">Config File Path</param>
        public static void AddOrUpdateConfig(string filePath = "appsettings.json")
        {
            var configFilePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, filePath);
            var _newParameters = ParameterDictionarySerializer.Deserialize(configFilePath);
            //替换原先的值_parameters
            if (_newParameters?.Count > 0)
            {
                foreach(var key in _newParameters.Keys)
                {
                    if (_parameters != null && _parameters.ContainsKey(key))
                    {
                        _parameters[key] = _newParameters[key];
                    }
                    else
                    {
                        if (_parameters == null)
                        {
                           _parameters = new ConcurrentDictionary<string, IParameter>();
                        }
                        _parameters.TryAdd(key, _newParameters[key]);
                    }
                }
            }
        }

        /// <summary>
        /// Add or Update DB Config Data
        /// </summary>
        /// <param name="json">DB Config Datas json</param>
        public static void AddOrUpdateDBConfig(string json)
        {
            var _newParameters = ParameterDictionarySerializer.Deserialize(json, 1);            
            //替换原先的值_parameters
            if (_newParameters?.Count > 0)
            {
                foreach(var key in _newParameters.Keys)
                {
                    if (_parameters != null && _parameters.ContainsKey(key))
                    {
                        _parameters[key] = _newParameters[key];
                    }
                    else
                    {
                        if (_parameters == null)
                        {
                           _parameters = new ConcurrentDictionary<string, IParameter>();
                        }
                        _parameters.TryAdd(key, _newParameters[key]);
                    }
                }
            }
        }

        /// <summary>
        /// Set parameter by specifed section name and parameter name
        /// </summary>
        public static bool SetParammeter(string sectionName, string paramName, IParameter value)
        {
            var key = sectionName + "_" + paramName;
            try
            {
                if (_parameters.ContainsKey(key))
                {
                    _parameters[key] = value;
                }
                else
                {
                    _parameters.TryAdd(key, value);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"EnvironmentConfigManager SetParammeter error, sectionName: {sectionName} - paramName: {paramName}, error: {ex}");   
                return false;
            }
            return true;
        }


        /// <summary>
        /// Get parameter by specifed section name and parameter name
        /// </summary>
        /// <typeparam name="T"></typeparam>
        public static T GetParammeter<T>(string sectionName, string paramName) where T : class
        {
            //var parameter = _parameters.FirstOrDefault(x => x.Section == sectionName && x.Name == paramName) as T;
            var key = sectionName + "_" + paramName;
            if (_parameters.ContainsKey(key))
            {
                var res = _parameters.TryGetValue(key, out IParameter _value);
                if (_value != null) 
                {
                    var parameter = _value as T;
                    if (parameter != null)
                    {
                        return parameter;
                    }
                    else
                    {
                        Logger.WriteLineWarn($"EnvironmentConfigManager GetParammeter parameter is null, sectionName: {sectionName} - paramName: {paramName} not set correct");
                    }
                }
                else
                {
                    Logger.WriteLineWarn($"EnvironmentConfigManager GetParammeter _value is null, sectionName: {sectionName} - paramName: {paramName} not set correct");
                }
            }
            else
            {
                Logger.WriteLineWarn($"EnvironmentConfigManager GetParammeter key is not exist, sectionName: {sectionName} - paramName: {paramName} not set correct");                    
            }
            throw new System.ArgumentException($"{sectionName} - {paramName} not set correct");
        }

        /// <summary>
        /// Get value by section name and item key
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="sectionName">The parameter section name</param>
        /// <param name="key">The parameter key</param>
        /// <param name="defaultValue">The parameter default value</param>
        /// <returns></returns>
        public static T GetValue<T>(string sectionName, string key, T defaultValue)
        {
            var t = typeof(T);
            //var itemValue = _parameters.FirstOrDefault(q => q.Section == sectionName && q.Name == key);
            var dicKey = sectionName + "_" + key;
            if (_parameters.ContainsKey(dicKey))
            {
                var res = _parameters.TryGetValue(dicKey, out IParameter itemValue);
                if (itemValue == null)
                {
                    return defaultValue;
                }
                object value = null;
                var type = itemValue.GetType();
                if (type == typeof(StringParameter))
                {
                    value = ((StringParameter)itemValue).Value;
                }
                if (type == typeof(IntParameter))
                {
                    value = ((IntParameter)itemValue).Value.ToString();
                }
                if (type == typeof(BoolParameter))
                {
                    value = ((BoolParameter)itemValue).Value.ToString();
                }
                if (type == typeof(EnumStringParameter))
                {
                    value = ((EnumStringParameter)itemValue).Source;
                }
                if (t == typeof(bool) && (value?.ToString() == "0" || value?.ToString() == "1"))
                {
                    return (T)(object)(value?.ToString() != "0");
                }
                return (T)Convert.ChangeType(value, t);
            }
            else
            {
                return defaultValue;
            }
        }

    }
}

