using System;
using System.Collections.Concurrent;
using WingServerCommon.Interfaces.OpLog;
using WingServerCommon.Log;
namespace WingServerCommon.Interfaces.Cache
{
    public class CacheManager<T> : IBaseCacheManager<T> where T : ICacheObject
    {
        protected readonly CacheList<T> CacheList = new CacheList<T>();

        private Func<IList<string>, IList<T>> _loadCachesFromDbWithIds;
        private Func<string, T> _loadCacheFromDbWithId;

        protected CacheManager()
        {
        }

        protected CacheManager(Func<string, T> loadCacheFromDbWithId, Func<IList<string>, IList<T>> loadCachesFromDbWithIds)
        {
            _loadCachesFromDbWithIds = loadCachesFromDbWithIds;
            _loadCacheFromDbWithId = loadCacheFromDbWithId;
        }

        /// <summary>
        /// The codes you removed when modify or delete
        /// </summary>
        /// <typeparam name="string"></typeparam>
        /// <returns></returns>
        private readonly List<string> _dirtyCodes = new List<string>();

        /// <summary>
        /// Register functions
        /// </summary>
        /// <param name="loadCacheFromDbWithId"></param>
        /// <param name="loadCachesFromDbWithIds"></param>
        public void RegisterDBFunction(Func<string, T> loadCacheFromDbWithId, Func<IList<string>, IList<T>> loadCachesFromDbWithIds)
        {
            _loadCachesFromDbWithIds = loadCachesFromDbWithIds;
            _loadCacheFromDbWithId = loadCacheFromDbWithId;
        }

        /// <summary>
        /// Load cache from db objects
        /// </summary>
        /// <param name="dbOjects">the db object list</param>
        public void LoadFromDbOject(IList<T> dbOjects)
        {
            CacheList.LoadFromDbOject(dbOjects);
        }

        /// <summary>
        /// Remove a cache record with specified document id
        /// </summary>
        /// <param name="documentId">The db collection document id</param>
        public void Remove(string documentId)
        {
            lock (_dirtyCodes)
            {
                var result = CacheList.Remove(documentId);
                if (result)
                {
                    if (!_dirtyCodes.Contains(documentId))
                    {
                        _dirtyCodes.Add(documentId);
                    }
                }
            }
        }

        /// <summary>
        /// Add a cache object to cache manager
        /// </summary>
        // <param name="cacheObj">A new cache object</param>    
        public void Add(T cacheObj)
        {
            CacheList.Add(cacheObj);
        }

        /// <summary>
        /// Remove many records from cache list; this function happened on sychronization actions
        /// </summary>
        /// <param name="ids"></param>
        public virtual void RemoveMany(CollectionDocumentChangedArgs args)
        {
            var ids = args.DocumentIds;
            CacheList.RemoveMany(ids);
            _dirtyCodes.AddRange(ids);
        }

        /// <summary>
        /// Get cache with specified document id
        /// </summary>
        /// <param name="documentId">the document id, the code in db</param>
        /// <typeparam name="T"></typeparam>
        /// <returns></returns>
        public T Get(string documentId)
        {
            var result = CacheList.Get(documentId);
            if (result == null)
            {
                //Load data from db if not exist in cache              
                result = _loadCacheFromDbWithId(documentId);
                if (result == null)
                {
                    Logger.WriteLineWarn($"not found from Db with id:{documentId}");
                    return result;
                }
                CacheList.Add(result);
                lock (_dirtyCodes)
                {
                    var code = result.Code;
                    if (_dirtyCodes.Contains(code))
                    {
                        _dirtyCodes.Remove(code);
                    }
                }

                return result;
            }

            return result;
        }

        /// <summary>
        /// Find values whith filters 
        /// </summary>
        /// <param name="predicate"></param>
        /// <returns></returns>
        public IList<T> Where(Func<T, bool> predicate)
        {
            lock (_dirtyCodes)
            {
                if (_dirtyCodes.Count > 0)
                {
                    //load cache with specified codes, then remove it
                    var caches = _loadCachesFromDbWithIds(_dirtyCodes);
                    foreach (var cache in caches)
                    {
                        CacheList.Add(cache);
                    }
                    _dirtyCodes.Clear();
                }
                return CacheList.Where(predicate);
            }
        }
    }
}