using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using WingServerCommon.Interfaces.Cache;

namespace WingServerCommon.Interfaces.Login
{
    public class LoginLock
    {
        public event EventHandler<LoginLockEventArgs> LoginLockInfoChanged;
        private int _loginLockTimes = 5;//默认一天内5次后锁定

        public LoginLock(int loginLockTimes)
        {
            _loginLockTimes = loginLockTimes;
        }

        protected LoginLockResult CheckLoginIsLocked(string code, DateTime lastLoginDate, int lastLoginTimes)
        {
            LoginLockResult loginLockTimesInfo = new LoginLockResult();
            if ((DateTime.UtcNow.Day - lastLoginDate.Day) > 0)
            {
                loginLockTimesInfo.Times = 1;
            }
            else
            {
                //当天次数
                if (lastLoginTimes >= _loginLockTimes - 1)
                {
                    loginLockTimesInfo.IsLocked = true;
                    loginLockTimesInfo.Times = lastLoginTimes + 1;
                }
                else
                {
                    if ((DateTime.UtcNow.Day - lastLoginDate.Day) > 0)
                    {
                        loginLockTimesInfo.Times = 1;
                    }
                    else
                    {
                        loginLockTimesInfo.Times = lastLoginTimes + 1;
                    }
                }
            }
            var remainingTimes = _loginLockTimes - loginLockTimesInfo.Times;
            if (remainingTimes < 0)
            {
                remainingTimes = 0;
            }
            loginLockTimesInfo.RemainingTimes = remainingTimes;
            LoginLockInfoChanged?.Invoke(this, new LoginLockEventArgs(code, loginLockTimesInfo.LoginDate, loginLockTimesInfo.Times));
            CacheMaintenance.Instance.Get<IUserInfoManager>().Remove(code);
            return loginLockTimesInfo;
        }

        public  bool GetLoginIsLocked(DateTime lastLoginDate, int lastLoginTimes)
        {
            return (DateTime.UtcNow.Day - lastLoginDate.Day) <= 0 && lastLoginTimes >= _loginLockTimes;
        }

    }
    /// <summary>
    /// 登录锁定管理额类
    /// </summary>
    public class UserLoginLock : LoginLock
    {
        public UserLoginLock(int loginLockTimes) : base(loginLockTimes)
        {

        }

        /// <summary>
        /// 检查返回登录错误信息方法
        /// </summary>
        /// <param name="userCode">user code</param>
        /// <returns>true: 登录锁定</returns>
        public LoginLockResult CheckUserLoginLocked(string userCode)
        {
            var user = CacheMaintenance.Instance.Get<IUserInfoManager>().Get(userCode);
            if (user != null)
            {
                var loginLockInfo = user.LoginLockInfo;
                if (loginLockInfo == null)
                {
                    loginLockInfo = new LoginLockInfo() { LoginDate = DateTime.UtcNow, Times = 0 };
                }
                return CheckLoginIsLocked(userCode, loginLockInfo.LoginDate, loginLockInfo.Times);
            }
            return null;
        }

        /// <summary>
        /// 获取是否账号已锁定
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public  bool GetLoginIsLocked(string code)
        {
            var user = CacheMaintenance.Instance.Get<IUserInfoManager>().Get(code);
            if (user != null)
            {
                var loginLockInfo = user.LoginLockInfo;
                if (loginLockInfo == null)
                {
                    loginLockInfo = new LoginLockInfo() { LoginDate = DateTime.UtcNow, Times = 0 };
                }
                return GetLoginIsLocked(loginLockInfo.LoginDate,loginLockInfo.Times);
            }
            return false;
        }

    }



    /// <summary>
    ///登录失败信息 
    /// </summary>
    public class LoginLockResult
    {

        /// <summary>
        /// 是否锁定
        /// </summary>
        /// <value></value>
        public bool IsLocked { get; set; } = false;

        /// <summary>
        /// 登录时间
        /// </summary>
        /// <value></value>
        public DateTime LoginDate { get; set; } = DateTime.UtcNow;

        /// <summary>
        /// 同一天的登录错误次数
        /// </summary>
        /// <value></value>
        public int Times { get; set; } = 0;

        /// <summary>
        /// 同一天剩余次数
        /// </summary>
        /// <value></value>
        public int RemainingTimes { get; set; } = 0;

    }

    /// <summary>
    /// 账号锁定信息事件
    /// </summary>
    public class LoginLockEventArgs : EventArgs
    {
        /// <summary>
        /// 账号编码
        /// </summary>
        /// <value></value>
        public string Code { get; set; }
        /// <summary>
        /// 登录时间
        /// </summary>
        /// <value></value>
        public DateTime LoginDate { get; set; }

        /// <summary>
        /// 同一天的登录错误次数
        /// </summary>
        /// <value></value>
        public int Times { get; set; }

        public LoginLockEventArgs(string code, DateTime loginDate, int times)
        {
            Code = code;
            LoginDate = loginDate;
            Times = times;
        }
    }
}